/* $Id: cliopen.c,v 0.7 1996/10/13 18:18:29 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            cliopen.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     A function for opening client side connections
 *                  for both tcp and udp.
 *                  Used internally.
 *
 *  WRITTEN BY      Sverre H. Huseby <sverrehu@ifi.uio.no>
 *
 **************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include "internal.h"
#include "shhnet.h"

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          _snCliOpenCommon
 *
 *  FUNCTION      Open a tcp- or udp-connection.
 *
 *  INPUT         type    the connection type: CONN_TCP or CONN_UDP
 *                host    a hostname, or a hostaddress in dotted quad
 *                        notation.
 *                service a service name, or a port number (ascii). If a
 *                        service name is given, it is mapped to a port
 *                        number.
 *                udpconnect
 *                        boolean: should connect be done for udp?
 *
 *  OUTPUT        sock    the socket referencing the connection.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   This function will create a socket and do the following
 *                depending on whether to connection type is tcp or udp:
 *                tcp     1. Connect to the given port on the given host.
 *                udp     1. Bind to some local port
 *                        2. Possibly `connect', depending on the
 *                           udpconnect-variable.
 */
SNErr
_snCliOpenCommon(SNConn type, const char *host, const char *service,
		 int udpconnect, int *sock)
{
    SNErr  err;
    SNAddr servip;
    SNPort   port;
    struct sockaddr_in servaddr;
    char   *servicename = NULL;

    switch (type) {
      case CONN_TCP:
	servicename = "tcp";
	break;
      case CONN_UDP:
	servicename = "udp";
	break;
    }
    
    memset(&servaddr, 0, sizeof(servaddr));
    servaddr.sin_family = AF_INET;

    /* get the port number corresponding to the given service. */
    if ((err = inServiceToPort(service, servicename, &port)) != SN_OK)
	return err;
    servaddr.sin_port = port;

    /* parse the server host address. */
    if ((err = inHostNameToAddr(host, &servip)) != SN_OK)
	return err;
    servaddr.sin_addr.s_addr = servip;

    /* create a socket. */
    if ((err = _snCreateSocket(type, 0, sock)) != SN_OK)
	return err;

    /* possibly connect the socket. */
    if (type != CONN_UDP || udpconnect)
	if (connect(*sock, (struct sockaddr *) &servaddr,
		    sizeof(servaddr)) < 0)
	    return _snError(SN_ERR_ERRNO, "connect");

    return SN_OK;
}
