/* $Id: in.c,v 0.6 1996/10/13 18:18:29 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            in.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     Functions for Internet addresses.
 *
 *  WRITTEN BY      Sverre H. Huseby <sverrehu@ifi.uio.no>
 *
 **************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "internal.h"
#include "shhnet.h"

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          inHostAddrToStr
 *
 *  FUNCTION      Return a dotted quad string from an ip-addres.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr inHostAddrToStr(SNAddr ip, char *s);
 *
 *  INPUT         in      internet address in network byte order.
 *
 *  OUTPUT        s       string to fill with name. must be at least
 *                        16 bytes long!
 *
 *  RETURNS       Error code.
 */
SNErr
inHostAddrToStr(SNAddr ip, char *s)
{
    struct in_addr ia;

    ia.s_addr = ip;
    strcpy(s, inet_ntoa(ia));
    return SN_OK;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          inHostAddrToName
 *
 *  FUNCTION      Return an official name from an ip-addres.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr inHostAddrToName(SNAddr ip, char *s, int len);
 *
 *  INPUT         ip      internet address in network byte order.
 *                len     length of buffer s.
 *
 *  OUTPUT        s       string to fill with name. Must be at least
 *                        16 bytes long!
 *
 *  RETURNS       Error code. This function does not follow the normal
 *                error handling: In case of error, the dotted quad is
 *                returned in instead of a name. No error message is
 *                displayed, and the program continues running.
 */
SNErr
inHostAddrToName(SNAddr ip, char *s, int len)
{
    struct in_addr ia;
    struct hostent *hp;

    ia.s_addr = ip;
    if ((hp = gethostbyaddr((char *) &ia, sizeof(ia), AF_INET)) == NULL) {
	inHostAddrToStr(ip, s);
	return SN_ERR_H_ERRNO;
    }
    strncpy(s, hp->h_name, len - 1);
    return SN_OK;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          inHostNameToAddr
 *
 *  FUNCTION      Return an ip-address from name or dotted quad string.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr inHostNameToAddr(const char *s, SNAddr *ip);
 *
 *  INPUT         s       string containing either a host name, or a
 *                        dotted quad representation of the ip-address.
 *
 *  OUTPUT        ip      internet address in network byte order.
 *
 *  RETURNS       Error code.
 */
SNErr
inHostNameToAddr(const char *s, SNAddr *ip)
{
    struct hostent *hp;

    /* check if dotted quad. */
    if ((*ip = inet_addr(s)) != -1)
	return SN_OK;

    /* not dotted quad. try to look up the name. */
    if ((hp = gethostbyname(s)) == NULL)
	return _snError(SN_ERR_H_ERRNO, s);
    *ip = *(SNAddr *)hp->h_addr;
    return SN_OK;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          inServiceToPort
 *
 *  FUNCTION      Convert a service name to a port number.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr inServiceToPort(const char *s, SNPort *port);
 *
 *  INPUT         s       string containing either a service name, or an
 *                        ascii representation of the port number.
 *                proto   either "tcp" or "udp".
 *
 *  OUTPUT        port    the port number in network byte order.
 *
 *  RETURNS       Error code.
 */
SNErr
inServiceToPort(const char *s, const char *proto, SNPort *port)
{
    struct servent *sp;

    /* check if ascii representation. */
    if (isdigit(*s)) {
	*port = htons(atoi(s));
	return SN_OK;
    }

    /* not ascii port number. look up the service. */
    if ((sp = getservbyname(s, proto)) == NULL)
	return _snError(SN_ERR_UNKNOWN_SERVICE, s);
    *port = sp->s_port;
    return SN_OK;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          inGetSockAddrPort
 *
 *  FUNCTION      Return the address and port describing a socket.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr inGetSockAddrPort(int sock, SNAddr *addr,
 *                                        SNPort *port);
 *
 *  INPUT         sock    the socket to get the address and port of.
 *
 *  OUTPUT        addr    the ip-address of the socket (network byte order).
 *                port    the port of the socket (network byte order).
 *
 *  RETURNS       Error code.
 */
SNErr
inGetSockAddrPort(int sock, SNAddr *addr, SNPort *port)
{
    int    alen;
    struct sockaddr_in saddr;

    alen = sizeof(saddr);
    if (getsockname(sock, (struct sockaddr *) &saddr, &alen) < 0)
	return _snError(SN_ERR_ERRNO, "getsockname");
    *addr = saddr.sin_addr.s_addr;
    *port = saddr.sin_port;
    return SN_OK;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          inGetSockStr
 *
 *  FUNCTION      Return an addr:port-string describing a socket.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr inGetSockStr(int sock, char *s, int lookupName);
 *
 *  INPUT         sock    the socket to get the name of.
 *                len     length of buffer s (at least 22).
 *                lookupName
 *                        map the ip-address to a name?
 *
 *  OUTPUT        s       string to fill with text. Must be at least
 *                        22 bytes long.
 *
 *  RETURNS       Error code.
 */
SNErr
inGetSockStr(int sock, char *s, int len, int lookupName)
{
    SNErr  err;
    int    alen;
    struct sockaddr_in addr;

    alen = sizeof(addr);
    if (getsockname(sock, (struct sockaddr *) &addr, &alen) < 0)
	return _snError(SN_ERR_ERRNO, "getsockname");
    if (lookupName) {
	if ((err = inHostAddrToName(addr.sin_addr.s_addr,
				    s, len - 6)) != SN_OK)
	    return err;
    } else {
	if ((err = inHostAddrToStr(addr.sin_addr.s_addr, s)) != SN_OK)
	    return err;
    }
    sprintf(s + strlen(s), ":%d", (int) ntohs(addr.sin_port));
    return SN_OK;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          inGetPeerAddrPort
 *
 *  FUNCTION      Return the address and port describing a peer.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr inGetPeerAddrPort(int sock, SNAddr *addr,
 *                                        SNPort *port);
 *
 *  INPUT         sock    the socket to get the peer address and port of.
 *
 *  OUTPUT        addr    the ip-address of the peer (network byte order).
 *                port    the port of the peer (network byte order).
 *
 *  RETURNS       Error code.
 */
SNErr
inGetPeerAddrPort(int sock, SNAddr *addr, SNPort *port)
{
    int    alen;
    struct sockaddr_in saddr;

    alen = sizeof(saddr);
    if (getpeername(sock, (struct sockaddr *) &saddr, &alen) < 0)
	return _snError(SN_ERR_ERRNO, "getpeername");
    *addr = saddr.sin_addr.s_addr;
    *port = saddr.sin_port;
    return SN_OK;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          inGetPeerStr
 *
 *  FUNCTION      Return an addr:port-string describing a peer.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr inGetPeerStr(int sock, char *s, int lookupName);
 *
 *  INPUT         sock    the socket to get the peer connection of.
 *                len     length of buffer s (at least 22).
 *                lookupName
 *                        map the ip-address to a name?
 *
 *  OUTPUT        s       string to fill with text. must be at least
 *                        22 bytes long.
 *
 *  RETURNS       Error code.
 */
SNErr
inGetPeerStr(int sock, char *s, int len, int lookupName)
{
    SNErr  err;
    int    alen;
    struct sockaddr_in addr;

    alen = sizeof(addr);
    if (getpeername(sock, (struct sockaddr *) &addr, &alen) < 0)
	return _snError(SN_ERR_ERRNO, "getpeername");
    if (lookupName) {
	if ((err = inHostAddrToName(addr.sin_addr.s_addr,
				    s, len - 6)) != SN_OK)
	    return err;
    } else {
	if ((err = inHostAddrToStr(addr.sin_addr.s_addr, s)) != SN_OK)
	    return err;
    }
    sprintf(s + strlen(s), ":%d", (int) ntohs(addr.sin_port));
    return SN_OK;
}
