/* $Id: internal.c,v 1.5 1996/10/13 18:18:30 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            internal.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     Used internally.
 *
 *  WRITTEN BY      Sverre H. Huseby <sverrehu@ifi.uio.no>
 *
 **************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>

#include "internal.h"
#include "shhnet.h"

/**************************************************************************
 *                                                                        *
 *                        P U B L I C    D A T A                          *
 *                                                                        *
 **************************************************************************/

SNErrorHandling _snErrorHandling = SN_REPORT_ERROR_AND_EXIT;
void (*_snErrorPrintf)(const char *format, ...) = NULL;



/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          _snCreateSocket
 *
 *  FUNCTION      Get a local socket, and possibly bind it.
 *
 *  INPUT         type    the connection type: CONN_TCP or CONN_UDP
 *                port    the port to bind to, network byte order.
 *                        a port number of 0 indicates "any port".
 *
 *  OUTPUT        sock    the socket.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   The socket is not bound if type is CONN_TCP and
 *                port is 0. This may be used when creating TCP client
 *                sockets.
 */
SNErr
_snCreateSocket(SNConn type, SNPort port, int *sock)
{
    struct sockaddr_in addr;
    int    socktype = 0;

    switch (type) {
      case CONN_TCP:
	socktype = SOCK_STREAM;
	break;
      case CONN_UDP:
	socktype = SOCK_DGRAM;
	break;
    }
    
    /* create a socket. */
    if ((*sock = socket(AF_INET, socktype, 0)) < 0)
	return _snError(SN_ERR_ERRNO, "socket");

    /* i'm not sure about this one. it was "if (port < 0)" */
    if (type == CONN_TCP && port == 0)
	return SN_OK;
    
    memset(&addr, 0, sizeof(addr));
    addr.sin_family = AF_INET;
    addr.sin_addr.s_addr = htonl(INADDR_ANY);
    addr.sin_port = port;

    if (bind(*sock, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
	close(*sock);
	return _snError(SN_ERR_ERRNO, "bind");
    }
    return SN_OK;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          _snError
 *
 *  FUNCTION      Handle an error as set up by snSetErrorHandling().
 *
 *  INPUT         err     the code of the error to handle.
 *                extra   any extra text to display along with the
 *                        error message, or NULL if no extra text should
 *                        be displayed. the error message will be of the
 *                        form "extra: error message" if extra is
 *                        not NULL.
 *
 *  RETURNS       An error code, or aborts the program if that's the
 *                wanted behaviour.
 *
 *  DESCRIPTION   This function is called for every error that occurs
 *                within the library functions. It possibly displays
 *                an error message, and possibly exits the program,
 *                according to the rules set by calling snSetErrorHandler().
 */
int
_snError(SNErr err, const char *extra)
{
    if (err == SN_OK)
	return err;
    if (_snErrorHandling == SN_RETURN_ERROR)
	return err;
    if (_snErrorHandling == SN_EXIT)
	exit(1);
    if (_snErrorPrintf) {
	if (extra)
	    _snErrorPrintf("%s: %s\n", extra, snErrToStr(err));
	else
	    _snErrorPrintf("%s\n", snErrToStr(err));
    } else {
	if (extra)
	    fprintf(stderr, "%s: %s\n", extra, snErrToStr(err));
	else
	    fprintf(stderr, "%s\n", snErrToStr(err));
    }
    if (_snErrorHandling == SN_REPORT_ERROR_AND_EXIT)
	exit(1);
    return err;
}
