/* $Id: midrgen.c,v 1.2 1996/10/13 18:18:30 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            midrgen.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     Machine Independent Data Representation, general
 *                  routines.
 *                  Routines for maintaining a buffer that may be transfered
 *                  across a network.
 *
 *                  All bytes are stored in network byte order (big
 *                  endian).
 *
 *                  Very little error checking is done. The user is supposed
 *                  to make sure the buffer doesn't overflow.
 *
 *  WRITTEN BY      Sverre H. Huseby <sverrehu@ifi.uio.no>
 *
 **************************************************************************/

#include <stdlib.h>

#include "shhmidr.h"

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          midrAlloc
 *
 *  FUNCTION      Allocate and initiate a MIDR-structure.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                MIDR *midrAlloc(size_t size);
 *
 *  INPUT         size    max number of bytes in one block.
 *
 *  RETURNS       Pointer to a new structure, or NULL on error.
 *                The structure should be deallocated using midrFree().
 *
 *  DESCRIPTION   The buffer is allocated according to the given size,
 *                and the index is set to the start.
 */
MIDR *
midrAlloc(size_t size)
{
    MIDR *ret;

    if ((ret = (MIDR *) malloc(size)) == (MIDR *) NULL)
	return ret;
    if ((ret->buff = (BYTE *) malloc(size)) == (BYTE *) NULL) {
	free(ret);
	return (MIDR *) NULL;
    }
    ret->size = size;
    ret->len = 0;
    ret->idx = 0;
    return ret;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          midrFree
 *
 *  FUNCTION      Release a MIDR-structure.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                void midrAlloc(MIDR *midr);
 *
 *  INPUT         midr    the structure to release. this may be NULL.
 *                        when this function returns, the variable
 *                        should no longer be used.
 *
 *  DESCRIPTION   Deallocates both the buffer and the structure.
 */
void
midrFree(MIDR *midr)
{
    if (midr == (MIDR *) NULL)
	return;
    free(midr->buff);
    free(midr);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          midrGetSize
 *
 *  FUNCTION      Get number of bytes allocated for the buffer.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                size_t midrGetSize(MIDR *midr);
 *
 *  INPUT         midr    the structure in question.
 *
 *  RETURNS       The number of bytes that will fit in the buffer. This
 *                is the same as the number passed to midrAlloc().
 */
size_t
midrGetSize(const MIDR *midr)
{
    return midr->size;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          midrGetBuff
 *
 *  FUNCTION      Get a pointer to the buffer in the given MIBR-structure.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                BYTE *midrGetBuff(MIDR *midr);
 *
 *  INPUT         midr    the structure in question.
 *
 *  RETURNS       A pointer to the data buffer inside the structure.
 *                If the contents are changed, midrSetLen() should be
 *                called to set the length accordingly.
 *
 *  DESCRIPTION   This function should be used when one wants to transfer
 *                the buffer contents to or from a destination or source.
 *                The buffer should not otherwise be changed.
 */
BYTE *
midrGetBuff(const MIDR *midr)
{
    return midr->buff;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          midrSetLen
 *
 *  FUNCTION      Set number of bytes in the buffer.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                void midrSetLen(MIDR *midr, size_t len);
 *
 *  INPUT         midr    the structure in question.
 *                len     number of bytes in the buffer.
 *
 *  DESCRIPTION   Use this function when the buffer is filled with data
 *                from a file or a network connection, to set the number
 *                of bytes read.
 */
void
midrSetLen(const MIDR *midr, size_t len)
{
    midr->len = len;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          midrGetLen
 *
 *  FUNCTION      Get number of bytes filled in.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                size_t midrGetLen(MIDR *midr);
 *
 *  INPUT         midr    the structure in question.
 *
 *  RETURNS       The number of relevant bytes in the buffer.
 *
 *  DESCRIPTION   This function should be used when one wants to transfer
 *                the buffer contents to a file or a network connection,
 *                to know how many bytes to send.
 */
size_t
midrGetLen(const MIDR *midr)
{
    return midr->idx;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          midrClear
 *
 *  FUNCTION      Empty the buffer of the MIDR structure.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                void midrClear(MIDR *midr);
 *
 *  INPUT         midr    the structure in question.
 *
 *  DESCRIPTION   Prepares for new input by resetting the index position
 *                and the length.
 */
void
midrClear(MIDR *midr)
{
    midr->len = 0;
    midr->idx = 0;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          midrRewind
 *
 *  FUNCTION      Initiate the MIDR-structure for rereading.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                void midrRewind(MIDR *midr);
 *
 *  INPUT         midr    the structure in question.
 *
 *  DESCRIPTION   Resets the index position only.
 */
void
midrRewind(MIDR *midr)
{
    midr->idx = 0;
}
