/* $Id: servhelp.c,v 1.1 1996/10/13 18:18:32 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            servhelp.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     Helper functions for daemons, not network related.
 *
 *  WRITTEN BY      Sverre H. Huseby <sverrehu@ifi.uio.no>
 *
 **************************************************************************/

#include <errno.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "internal.h"
#include "shhnet.h"

/**************************************************************************
 *                                                                        *
 *                   P R I V A T E    F U N C T I O N S                   *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          _snSigChildHandler
 *
 *  FUNCTION      Handle signals raised when a child is dying.
 *
 *  INPUT         s       the signal number. not used.
 *
 *  DESCRIPTION   This function is included to avoid zombies. When a child
 *                dies, it won't fully die untill the parent has signed for
 *                it using some kind of wait(). This handler is called when
 *                a child is about to die, so including a wait here will
 *                solve the problem.
 */
static void
_snSigChildHandler(int s)
{
    int olderrno, status;

    olderrno = errno;
#ifdef WAIT_ANY
    waitpid(WAIT_ANY, &status, WNOHANG);
#else
    /* WAIT_ANY was undefined on SunOS 4.1.3_U1 */
    waitpid(-1, &status, WNOHANG);
#endif
    errno = olderrno;
}



/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          snServerSetupSigHandlers
 *
 *  FUNCTION      Install signal handlers.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr snServerSetupSigHandlers(SNSigHandler termHandler,
 *                                               SNSigHandler hupHandler);
 *
 *  INPUT         termHandler
 *                        function handling several kinds of signal
 *                        intended to abort the program. if NULL, no
 *                        handler is installed for these signals.
 *                hupHandler
 *                        function handling SIGHUP, the hangup signal.
 *                        this signal is often used by daemon processes
 *                        to indicate that setup files should be reread.
 *                        if NULL, the SIGHUP signal is treated as the
 *                        other signals that abort the program; the
 *                        function given by termHandler is installed.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   In addition to termination handlers and a hangup handler,
 *                a SIGCHLD handler is installed. Servers typically
 *                spawns children to handle connections, and special
 *                handling should be done when children terminates to
 *                avoid zombie processes. The installed handler fixes
 *                this.
 */
SNErr
snServerSetupSigHandlers(SNSigHandler termHandler, SNSigHandler hupHandler)
{
    struct sigaction sact;

    sigemptyset(&sact.sa_mask);
    /* Signals are troublesome. On POSIX and System V, the default is
     * to not restart system calls that are interrupted by signals,
     * but rather make them fail. Thus we must specify that we want
     * the calls to be restarted. On BSD however, the default is to
     * restart the calls, so we should do nothing special. */
#ifdef SA_RESTART
    /* probably POSIX or System V. */
    sact.sa_flags = SA_RESTART;
#else
    /* probably BSD. */
    sact.sa_flags = 0;
#endif

    /* dead child signal. */
    sact.sa_handler = _snSigChildHandler;
    if (sigaction(SIGCHLD, &sact, NULL) < 0)
	return _snError(SN_ERR_ERRNO, "sicaction");

    /* signals for program termination. */
    if (termHandler) {
	sact.sa_handler = termHandler;
	if (sigaction(SIGINT, &sact, NULL) < 0
	    || sigaction(SIGTERM, &sact, NULL) < 0
	    || sigaction(SIGQUIT, &sact, NULL) < 0
	    || sigaction(SIGABRT, &sact, NULL) < 0
	    || (!hupHandler && sigaction(SIGHUP, &sact, NULL) < 0))
	    return _snError(SN_ERR_ERRNO, "sicaction");
    }

    /* hangup signal. */
    if (hupHandler) {
	sact.sa_handler = hupHandler;
	if (sigaction(SIGHUP, &sact, NULL) < 0)
	    return _snError(SN_ERR_ERRNO, "sicaction");
    }
    return SN_OK;
}
