/* $Id: udpio.c,v 0.5 1996/10/13 18:18:34 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            udpio.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     Functions handling IO on a udp-connection.
 *
 *  WRITTEN BY      Sverre H. Huseby <sverrehu@ifi.uio.no>
 *
 **************************************************************************/

#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include "internal.h"
#include "shhnet.h"

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          udpRead
 *
 *  FUNCTION      Read from a udp-connection.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr udpRead(int sock, char *buf,
 *                              size_t count, size_t *nread);
 *
 *  INPUT         sock    descriptor of udp-stream.
 *                buf     pointer to buffer that will receive the data.
 *                count   number of bytes to read.
 *
 *  OUTPUT        nread   number of bytes read.
 *
 *  RETURNS       Error code.
 */
SNErr
udpRead(int sock, char *buf, size_t count, size_t *nread)
{
    long n;

    if ((n = recv(sock, buf, count, 0)) < 0)
	return _snError(SN_ERR_ERRNO, "recv");
    *nread = n;
    return SN_OK;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          udpWrite
 *
 *  FUNCTION      Write to a udp-connection.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr udpWrite(int sock, char *buf,
 *                               size_t count, size_t *nwritten);
 *
 *  INPUT         sock    descriptor of udp-stream.
 *                buf     pointer to buffer that contains the data.
 *                count   number of bytes to write.
 *
 *  OUTPUT        nwritten
 *                        number of bytes written.
 *
 *  RETURNS       Error code.
 */
SNErr
udpWrite(int sock, const char *buf, size_t count, size_t *nwritten)
{
    long n;

    if ((n = send(sock, buf, count, 0)) < 0)
	return _snError(SN_ERR_ERRNO, "send");
    *nwritten = n;
    return SN_OK;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          udpReadFrom
 *
 *  FUNCTION      Read from a specified udp-connection.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr udpReadFrom(int sock, char *buf, size_t count,
 *                                  SNAddr *addr, SNPort port, size_t *nread);
 *
 *  INPUT         sock    descriptor of udp-stream.
 *                buf     pointer to buffer that will receive the data.
 *                count   number of bytes to read.
 *
 *  OUTPUT        addr    the address read from (network byte order).
 *                port    the port read from (network byte order).
 *                nread   number of bytes read.
 *
 *  RETURNS       Error code.
 */
SNErr
udpReadFrom(int sock, char *buf, size_t count,
	    SNAddr *addr, SNPort *port, size_t *nread)
{
    int    n, len;
    struct sockaddr_in sa;

    len = sizeof(sa);
    if ((n = recvfrom(sock, buf, count, 0, (struct sockaddr *) &sa, &len)) < 0)
	return _snError(SN_ERR_ERRNO, "recvfrom");
    *addr = sa.sin_addr.s_addr;
    *port = sa.sin_port;
    *nread = n;
    return SN_OK;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          udpWriteTo
 *
 *  FUNCTION      Write to a specified udp-connection.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr udpWriteTo(int sock, char *buf, size_t count,
 *                                 SNAddr addr, SNPort port, size_t *nwritten);
 *
 *  INPUT         sock    descriptor of udp-stream.
 *                buf     pointer to buffer that contains the data.
 *                count   number of bytes to write.
 *                addr    the address to write to (network byte order).
 *                port    the port to write to (network byte order).
 *
 *  OUTPUT        nwritten
 *                        number of bytes written.
 *
 *  RETURNS       Error code.
 */
SNErr
udpWriteTo(int sock, const char *buf, size_t count,
	   SNAddr addr, SNPort port, size_t *nwritten)
{
    int    n;
    struct sockaddr_in sa;

    sa.sin_family = AF_INET;
    sa.sin_addr.s_addr = addr;
    sa.sin_port = port;
    if ((n = sendto(sock, buf, count, 0,
		    (struct sockaddr *) &sa, sizeof(sa))) < 0)
	return _snError(SN_ERR_ERRNO, "sendto");
    *nwritten = n;
    return SN_OK;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          udpClose
 *
 *  FUNCTION      Close a udp-connection.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr udpClose(int sock);
 *
 *  INPUT         sock    descriptor of udp-connection.
 *
 *  RETURNS       Error code.
 */
SNErr
udpClose(int sock)
{
    if (close(sock) < 0)
	return _snError(SN_ERR_ERRNO, "close");
    return SN_OK;
}
